@@ -1,20 +1,21 @@
-from typing import Dict, Any, Tuple, Type
+from typing import Dict, Any, Tuple, Type, Optional, Union, List
 import types
 import sys
 
+from bugsnag.breadcrumbs import BreadcrumbType, OnBreadcrumbCallback
+from bugsnag.feature_flags import FeatureFlag
 from bugsnag.configuration import RequestConfiguration
 from bugsnag.client import Client
 
-import bugsnag
-
 default_client = Client()
 configuration = default_client.configuration
+logger = configuration.logger
 ExcInfoType = Tuple[Type, Exception, types.TracebackType]
 
 
 __all__ = ('configure', 'configure_request', 'add_metadata_tab',
            'clear_request_config', 'notify', 'start_session', 'auto_notify',
-           'auto_notify_exc_info', 'before_notify')
+           'auto_notify_exc_info', 'before_notify', 'leave_breadcrumb')
 
 
 def configure(**options):
@@ -70,8 +71,10 @@ def notify(exception: BaseException, **options):
             except Exception:
                 value = '[BADENCODING]'
 
-            bugsnag.logger.warning('Coercing invalid notify()'
-                                   ' value to RuntimeError: %s' % value)
+            default_client.configuration.logger.warning(
+                'Coercing invalid notify() value to RuntimeError: %s' % value
+            )
+
             exception = RuntimeError(value)
 
         default_client.notify(exception, **options)
@@ -100,7 +103,7 @@ def auto_notify(exception: BaseException, **options):
         )
 
 
-def auto_notify_exc_info(exc_info: ExcInfoType = None, **options):
+def auto_notify_exc_info(exc_info: Optional[ExcInfoType] = None, **options):
     """
     Notify bugsnag of a exc_info tuple if auto_notify is enabled
     """
@@ -126,3 +129,54 @@ def before_notify(callback):
     This can be used to alter the event before sending it to Bugsnag.
     """
     configuration.middleware.before_notify(callback)
+
+
+def leave_breadcrumb(
+    message: str,
+    metadata: Dict[str, Any] = {},
+    type: BreadcrumbType = BreadcrumbType.MANUAL
+) -> None:
+    default_client.leave_breadcrumb(message, metadata, type)
+
+
+def add_on_breadcrumb(on_breadcrumb: OnBreadcrumbCallback) -> None:
+    """
+    Add a callback to be called each time a breadcrumb is left
+
+    This can be used to alter the breadcrumb, or discard it by returning False
+    """
+    default_client.add_on_breadcrumb(on_breadcrumb)
+
+
+def remove_on_breadcrumb(on_breadcrumb: OnBreadcrumbCallback) -> None:
+    """
+    Remove an existing on_breadcrumb callback
+    """
+    default_client.remove_on_breadcrumb(on_breadcrumb)
+
+
+def _auto_leave_breadcrumb(
+    message: str,
+    metadata: Dict[str, Any],
+    type: BreadcrumbType
+) -> None:
+    default_client._auto_leave_breadcrumb(message, metadata, type)
+
+
+def add_feature_flag(
+    name: Union[str, bytes],
+    variant: Union[None, str, bytes] = None
+) -> None:
+    default_client.add_feature_flag(name, variant)
+
+
+def add_feature_flags(feature_flags: List[FeatureFlag]) -> None:
+    default_client.add_feature_flags(feature_flags)
+
+
+def clear_feature_flag(name: Union[str, bytes]) -> None:
+    default_client.clear_feature_flag(name)
+
+
+def clear_feature_flags() -> None:
+    default_client.clear_feature_flags()
