@@ -2,11 +2,21 @@ from functools import wraps, partial
 import inspect
 from json import JSONEncoder
 from threading import local as threadlocal
-from typing import Tuple, Optional
+from typing import AnyStr, Tuple, Optional
 import warnings
 import copy
+import logging
+from datetime import datetime, timedelta
+from urllib.parse import urlparse, urlunsplit, parse_qs
+
+
+try:
+    from os import PathLike
+except ImportError:
+    # PathLike was added in Python 3.6 so fallback to PurePath on Python 3.5 as
+    # all builtin Path objects inherit from PurePath
+    from pathlib import PurePath as PathLike  # type: ignore
 
-import bugsnag
 
 MAX_PAYLOAD_LENGTH = 128 * 1024
 MAX_STRING_LENGTH = 1024
@@ -20,8 +30,10 @@ class SanitizingJSONEncoder(JSONEncoder):
     A JSON encoder which handles filtering and conversion from JSON-
     incompatible types to strings.
 
+    >>> import logging
     >>> from json import loads
-    >>> encoder = SanitizingJSONEncoder(keyword_filters=['bananas'])
+    >>> logger = logging.getLogger(__name__)
+    >>> encoder = SanitizingJSONEncoder(logger, keyword_filters=['bananas'])
     >>> items = loads(encoder.encode(FilterDict({'carrots': 4, 'bananas': 5})))
     >>> items['bananas']
     '[FILTERED]'
@@ -33,8 +45,10 @@ class SanitizingJSONEncoder(JSONEncoder):
     recursive_value = '[RECURSIVE]'
     unencodeable_value = '[BADENCODING]'
 
-    def __init__(self, keyword_filters=None, **kwargs):
+    def __init__(self, logger: logging.Logger, keyword_filters=None, **kwargs):
+        self.logger = logger
         self.filters = list(map(str.lower, keyword_filters or []))
+        self.bytes_filters = [x.encode('utf-8') for x in self.filters]
         super(SanitizingJSONEncoder, self).__init__(**kwargs)
 
     def encode(self, obj):
@@ -70,13 +84,15 @@ class SanitizingJSONEncoder(JSONEncoder):
 
             clean_dict = {}
             for key, value in obj.items():
-                is_string = isinstance(key, str)
-                if is_string and any(f in key.lower() for f in self.filters):
+                if self._should_filter(key):
                     clean_dict[key] = self.filtered_value
                 else:
                     clean_dict[key] = self.filter_string_values(
                         value, ignored, seen)
 
+            # Only ignore whilst encoding children
+            ignored.remove(id(obj))
+
             return clean_dict
 
         return obj
@@ -93,7 +109,7 @@ class SanitizingJSONEncoder(JSONEncoder):
                 return str(obj)
 
         except Exception:
-            bugsnag.logger.exception('Could not add object to payload')
+            self.logger.exception('Could not add object to payload')
             return self.unencodeable_value
 
     def _sanitize(self, obj, trim_strings, ignored=None, seen=None):
@@ -115,16 +131,25 @@ class SanitizingJSONEncoder(JSONEncoder):
         if id(obj) in ignored:
             return self.recursive_value
         elif isinstance(obj, dict):
-            ignored.add(id(obj))
             seen.append(obj)
-            return self._sanitize_dict(obj, trim_strings, ignored, seen)
+
+            ignored.add(id(obj))
+            sanitized = self._sanitize_dict(obj, trim_strings, ignored, seen)
+            # Only ignore whilst encoding children
+            ignored.remove(id(obj))
+
+            return sanitized
         elif isinstance(obj, (set, tuple, list)):
-            ignored.add(id(obj))
             seen.append(obj)
+
+            ignored.add(id(obj))
             items = []
             for value in obj:
                 items.append(
                         self._sanitize(value, trim_strings, ignored, seen))
+            # Only ignore whilst encoding children
+            ignored.remove(id(obj))
+
             return items
         elif trim_strings and isinstance(obj, str):
             return obj[:MAX_STRING_LENGTH]
@@ -141,7 +166,7 @@ class SanitizingJSONEncoder(JSONEncoder):
                 key = str(key, encoding='utf-8', errors='replace')
                 clean_dict[key] = clean_value
             except Exception:
-                bugsnag.logger.exception(
+                self.logger.exception(
                     'Could not add sanitize key for dictionary, '
                     'dropping value.')
         if isinstance(key, str):
@@ -150,7 +175,7 @@ class SanitizingJSONEncoder(JSONEncoder):
             try:
                 clean_dict[str(key)] = clean_value
             except Exception:
-                bugsnag.logger.exception(
+                self.logger.exception(
                     'Could not add sanitize key for dictionary, '
                     'dropping value.')
 
@@ -171,6 +196,17 @@ class SanitizingJSONEncoder(JSONEncoder):
 
         return clean_dict
 
+    def _should_filter(self, key):
+        if isinstance(key, str):
+            key_lower = key.lower()
+            return any(f in key_lower for f in self.filters)
+
+        if isinstance(key, bytes):
+            key_lower = key.lower()
+            return any(f in key_lower for f in self.bytes_filters)
+
+        return False
+
 
 class FilterDict(dict):
     """
@@ -225,24 +261,29 @@ def is_json_content_type(value: str) -> bool:
     return type == 'application' and (subtype == 'json' or suffix == 'json')
 
 
-def fully_qualified_class_name(obj):
+_ignore_modules = ('__main__', 'builtins')
+
+
+def partly_qualified_class_name(obj):
     module = inspect.getmodule(obj)
-    if module is not None and module.__name__ != "__main__":
-        return module.__name__ + "." + obj.__class__.__name__
-    else:
+
+    if module is None or module.__name__ in _ignore_modules:
         return obj.__class__.__name__
 
+    return module.__name__ + '.' + obj.__class__.__name__
 
-def package_version(package_name):
-    try:
-        import pkg_resources
-    except ImportError:
-        return None
+
+def fully_qualified_class_name(obj):
+    module = inspect.getmodule(obj)
+    if hasattr(obj.__class__, "__qualname__"):
+        qualified_name = obj.__class__.__qualname__
     else:
-        try:
-            return pkg_resources.get_distribution(package_name).version
-        except pkg_resources.DistributionNotFound:
-            return None
+        qualified_name = obj.__class__.__name__
+
+    if module is None or module.__name__ in _ignore_modules:
+        return qualified_name
+
+    return module.__name__ + '.' + qualified_name
 
 
 def _validate_setter(types, func, should_error=False):
@@ -273,6 +314,8 @@ validate_required_str_setter = partial(_validate_setter, (str,),
                                        should_error=True)
 validate_bool_setter = partial(_validate_setter, (bool,))
 validate_iterable_setter = partial(_validate_setter, (list, tuple))
+validate_int_setter = partial(_validate_setter, (int,))
+validate_path_setter = partial(_validate_setter, (str, PathLike))
 
 
 class ThreadContextVar:
@@ -313,3 +356,108 @@ class ThreadContextVar:
 
     def set(self, new_value):
         setattr(ThreadContextVar.local_context(), self.name, new_value)
+
+
+def sanitize_url(url: AnyStr, config) -> Optional[str]:
+    try:
+        if isinstance(url, str):
+            url_str = url
+        else:
+            url_str = str(url, encoding='utf-8', errors='replace')
+
+        parsed = urlparse(url_str)
+
+        # if there's no query string there's nothing to redact
+        if not parsed.query:
+            return url_str
+
+        url_without_query = urlunsplit(
+            # urlunsplit always requires 5 elements in this tuple
+            (parsed.scheme, parsed.netloc, parsed.path, None, None)
+        ).strip()
+
+        query_parameters = parse_qs(parsed.query)
+    except Exception:
+        # if we can't parse the url or query string then we can't know if
+        # there's anything to redact, so have to omit the URL entirely
+        return None
+
+    encoder = SanitizingJSONEncoder(config.logger, config.params_filters)
+    redacted_parameter_dict = encoder.filter_string_values(query_parameters)
+
+    filtered_value = SanitizingJSONEncoder.filtered_value
+    redacted_parameters = []
+
+    for key, values in redacted_parameter_dict.items():
+        # if "values" has been redacted it's a string, otherwise it's a list
+        if values == filtered_value:
+            redacted_parameters.append(key + "=" + values)
+        else:
+            for value in values:
+                redacted_parameters.append(key + "=" + value)
+
+    return url_without_query + "?" + "&".join(redacted_parameters)
+
+
+def remove_query_from_url(url: AnyStr) -> Optional[AnyStr]:
+    try:
+        parsed = urlparse(url)
+
+        url_without_query = urlunsplit(
+            # urlunsplit always requires 5 elements in this tuple
+            (parsed.scheme, parsed.netloc, parsed.path, None, None)
+        ).strip()
+    except Exception:
+        return None
+
+    # If the returned url is empty then it did not have any of the components
+    # we are interested in, so return None to indicate failure
+    if not url_without_query:
+        return None
+
+    return url_without_query
+
+
+# to_rfc3339: format a datetime instance to match to_rfc3339/iso8601 with
+# milliseconds precision
+# Python can do this natively from version 3.6, but we need to include a
+# fallback implementation for Python 3.5
+try:
+    # this will raise if 'timespec' isn't supported
+    datetime.utcnow().isoformat(timespec='milliseconds')  # type: ignore
+
+    def to_rfc3339(dt: datetime) -> str:
+        return dt.isoformat(timespec='milliseconds')  # type: ignore
+
+except Exception:
+    def _get_timezone_offset(dt: datetime) -> str:
+        if dt.tzinfo is None:
+            return ''
+
+        utc_offset = dt.tzinfo.utcoffset(dt)
+
+        if utc_offset is None:
+            return ''
+
+        sign = '+'
+
+        if utc_offset.days < 0:
+            sign = '-'
+            utc_offset = -utc_offset
+
+        hours_offset, minutes = divmod(utc_offset, timedelta(hours=1))
+        minutes_offset, seconds = divmod(minutes, timedelta(minutes=1))
+
+        return '{:s}{:02d}:{:02d}'.format(sign, hours_offset, minutes_offset)
+
+    def to_rfc3339(dt: datetime) -> str:
+        return '{:04d}-{:02d}-{:02d}T{:02d}:{:02d}:{:02d}.{:03d}{:s}'.format(
+            dt.year,
+            dt.month,
+            dt.day,
+            dt.hour,
+            dt.minute,
+            dt.second,
+            int(dt.microsecond / 1000),
+            _get_timezone_offset(dt)
+        )
