@@ -1,8 +1,13 @@
 from webob import Request
 import sys
+from typing import Dict
 
 import bugsnag
+from bugsnag.context import create_new_context
 from bugsnag.wsgi import request_path
+from bugsnag.breadcrumbs import BreadcrumbType
+from bugsnag.legacy import _auto_leave_breadcrumb
+from bugsnag.utils import remove_query_from_url
 
 # Attempt to import bottle for runtime version report, but only if already
 # in use in app
@@ -55,9 +60,18 @@ class WrappedWSGIApp:
         self.environ = environ
 
         bugsnag.configure_request(wsgi_environ=self.environ)
+        create_new_context()
+
         try:
             if bugsnag.configuration.auto_capture_sessions:
                 bugsnag.start_session()
+
+            _auto_leave_breadcrumb(
+                'http request',
+                _get_breadcrumb_metadata(environ),
+                BreadcrumbType.NAVIGATION
+            )
+
             self.app = application(environ, start_response)
         except Exception as e:
             bugsnag.auto_notify(
@@ -92,6 +106,18 @@ class WrappedWSGIApp:
             bugsnag.clear_request_config()
 
 
+def _get_breadcrumb_metadata(environ) -> Dict[str, str]:
+    metadata = {}
+
+    if 'PATH_INFO' in environ:
+        metadata['to'] = environ['PATH_INFO']
+
+    if 'HTTP_REFERER' in environ:
+        metadata['from'] = remove_query_from_url(environ['HTTP_REFERER'])
+
+    return metadata
+
+
 class BugsnagMiddleware:
     """
     Notifies Bugsnag on any unhandled exception that happens while processing
